# 杂项功能
# 2023年1月1日
# 末心
# 社会主义核心价值观编码
# 需要将Python目录下>lib>subprocess.py的765行附近的encoding的默认None值修改为utf-8即可。
import os
import execjs  # PyExecJS
from qsnctf.auxiliary import js_from_file
import urllib.parse
import re
import html
import zipfile
import queue
import time
import threading
import uuid
from qsnctf.auxiliary import read_file_to_list, is_http_or_https_url, normalize_url


def get_uuid():
    return str(uuid.uuid4())


def Chinese_socialism_encode(string):
    # 社会主义核心价值观编码
    package_path = os.path.abspath(os.path.dirname(__file__))
    file_path = os.path.join(package_path, 'plugin', 'js', 'cvencode.js')
    content = execjs.compile(js_from_file(file_path))
    return str(content.call('encode', string))


def Chinese_socialism_decode(string):
    # 社会主义核心价值观解码
    package_path = os.path.abspath(os.path.dirname(__file__))
    file_path = os.path.join(package_path, 'plugin', 'js', 'cvencode.js')
    content = execjs.compile(js_from_file(file_path))
    return str(content.call('decode', string))


def string_reverse(string):
    """
    直接逆向
    主要应用场合为文件的Hex的转换
    :param string: abc123
    :return: 321cba
    """
    return string[::-1]


def string_reverse_step(string, step):
    """
    步长为step的逆向(自定义步长)
    主要应用场合为文件的Hex的转换
    :param string: abc123
    :param step: 步长
    :return: ba1c32
    """
    lst = [c for c in string]
    result = ''
    for i in range(0, len(lst), step):
        result += ''.join(lst[i:i + step][::-1])
    return result


def string_reverse_step2(string):
    """
    步长为2的逆向
    主要应用场合为文件的Hex的转换
    :param string: abc123
    :return: ba1c32
    """
    string_reverse_step(string, 2)


def url_encode(string):
    # url编码
    return urllib.parse.quote(string)


def url_decode(string):
    # url解码
    return urllib.parse.unquote(string)


def xor_list(lt_data, lt_root):
    # 异或 lt_root直接传列表过来，这样也可以字符串之间互相异或
    decrypted = ""
    for i in range(min(len(lt_data), len(lt_root))):
        decrypted += chr(ord(lt_data[i]) ^ ord(lt_root[i]))
    return decrypted


def string_split(s):
    # 如果字符串中包含逗号，就按照逗号分割
    if ',' in s:
        return s.split(',')
    # 如果包含空格按照空格分割
    elif ' ' in s:
        return s.split()
    # 否则按照字符方式分割
    else:
        r_list = []
        for str in s:
            r_list.append(str)
        return r_list


def ord_to_str(ord):
    return chr(int(ord))


def ord_list_to_str_list(ord_list):
    r_list = []
    for ords in ord_list:
        r_list.append(ord_to_str(ords))
    return r_list


def ord_str_to_str(ord_str):
    r_text = ''
    ord_list = string_split(ord_str)
    for ords in ord_list:
        r_text += ','.join(ord_to_str(ords))
    return r_text


def chr_to_ord(char):
    return ord(char)


def chr_list_to_ord_list(chr_list):
    r_list = []
    for char in chr_list:
        r_list.append(chr_to_ord(char))
    return r_list


def chr_str_to_ord_str(chr_str):
    r_text = ''
    char = string_split(chr_str)
    for chars in char:
        r_text += str(chr_to_ord(chars)) + ","
    return r_text[:-1]


def search_flag(text, flag_prefix='flag|qsnctf|ctf'):
    """
    :param text: search string
    :param flag_prefix: flag prefix eg: flag|qsnctf|ctf
    :return: flag{xxxxxxx}
    """
    # pattern = f'({flag_prefix})' + r'\{[\w]+\}'
    pattern = r'(' + flag_prefix + ')\{.+\}'
    match = re.search(pattern, text)
    if match:
        result = match.group(0)
        return result
    else:
        return False


def baijiaxing_encode(source_text):
    CODE = {
        "赵": "0", "钱": "1", "孙": "2", "李": "3", "周": "4", "吴": "5", "郑": "6", "王": "7", "冯": "8",
        "陈": "9",
        "褚": "a", "卫": "b", "蒋": "c", "沈": "d", "韩": "e", "杨": "f", "朱": "g", "秦": "h", "尤": "i",
        "许": "j",
        "何": "k", "吕": "l", "施": "m", "张": "n", "孔": "o", "曹": "p", "严": "q", "华": "r", "金": "s",
        "魏": "t",
        "陶": "u", "姜": "v", "戚": "w", "谢": "x", "邹": "y", "喻": "z", "福": "A", "水": "B", "窦": "C",
        "章": "D",
        "云": "E", "苏": "F", "潘": "G", "葛": "H", "奚": "I", "范": "J", "彭": "K", "郎": "L", "鲁": "M",
        "韦": "N",
        "昌": "O", "马": "P", "苗": "Q", "凤": "R", "花": "S", "方": "T", "俞": "U", "任": "V", "袁": "W",
        "柳": "X",
        "唐": "Y", "罗": "Z", "薛": ".", "伍": "-", "余": "_", "米": "+", "贝": "=", "姚": "/", "孟": "?",
        "顾": "#",
        "尹": "%", "江": "&", "钟": "*"
    }
    source_text = re.sub('[\u4e00-\u9fa5]', '', source_text)
    # source_text = source_text.replace( r"/ ^\s\s * /", '').replace( r"/\s\s *$ /", '')
    CODE = dict((value, key) for key, value in CODE.items())
    cc = [CODE[i] for i in source_text]
    dd = ''.join(cc)
    return dd


def baijiaxing_decode(source_text):
    CODE = {
        "赵": "0", "钱": "1", "孙": "2", "李": "3", "周": "4", "吴": "5", "郑": "6", "王": "7", "冯": "8",
        "陈": "9",
        "褚": "a", "卫": "b", "蒋": "c", "沈": "d", "韩": "e", "杨": "f", "朱": "g", "秦": "h", "尤": "i",
        "许": "j",
        "何": "k", "吕": "l", "施": "m", "张": "n", "孔": "o", "曹": "p", "严": "q", "华": "r", "金": "s",
        "魏": "t",
        "陶": "u", "姜": "v", "戚": "w", "谢": "x", "邹": "y", "喻": "z", "福": "A", "水": "B", "窦": "C",
        "章": "D",
        "云": "E", "苏": "F", "潘": "G", "葛": "H", "奚": "I", "范": "J", "彭": "K", "郎": "L", "鲁": "M",
        "韦": "N",
        "昌": "O", "马": "P", "苗": "Q", "凤": "R", "花": "S", "方": "T", "俞": "U", "任": "V", "袁": "W",
        "柳": "X",
        "唐": "Y", "罗": "Z", "薛": ".", "伍": "-", "余": "_", "米": "+", "贝": "=", "姚": "/", "孟": "?",
        "顾": "#",
        "尹": "%", "江": "&", "钟": "*"
    }
    source_text = re.sub('[^\u4e00-\u9fa5]+', '', source_text)
    # source_text = source_text.replace( r"/ ^\s\s * /", '').replace( r"/\s\s *$ /", '')
    cc = [CODE[i] for i in source_text]
    dd = ''.join(cc)
    if dd:
        return dd
    else:
        return '解码失败'


def qwerty_encode(source_text):
    str1 = "qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM"
    str2 = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
    result_text = ""
    for s in source_text:
        if s in str1:
            if s != ' ':
                result_text = result_text + str1[str2.index(s)]
            else:
                result_text = result_text + ' '
        else:
            return 'Qwerty只能对字母编码!'
    return result_text


def qwerty_decode(source_text):
    try:
        letter = {
            'q': 'a', 'w': 'b', 'e': 'c', 'r': 'd', 't': 'e', 'y': 'f', 'u': 'g',
            'i': 'h', 'o': 'i', 'p': 'j', 'a': 'k', 's': 'l', 'd': 'm', 'f': 'n',
            'g': 'o', 'h': 'p', 'j': 'q', 'k': 'r', 'l': 's', 'z': 't',
            'x': 'u', 'c': 'v', 'v': 'w', 'b': 'x', 'n': 'y', 'm': 'z',
            'Q': 'A', 'W': 'B', 'E': 'C', 'R': 'D', 'T': 'E', 'Y': 'F', 'U': 'G',
            'I': 'H', 'O': 'I', 'P': 'J', 'A': 'K', 'S': 'L', 'D': 'M', 'F': 'N',
            'G': 'O', 'H': 'P', 'J': 'Q', 'K': 'R', 'L': 'S', 'Z': 'T',
            'X': 'U', 'C': 'V', 'V': 'W', 'B': 'X', 'N': 'Y', 'M': 'Z',
        }
        result_text = ''
        for i in range(0, len(source_text)):
            if source_text[i] != ' ':
                result_text = result_text + letter.get(source_text[i])
            else:
                result_text = result_text + ' '
    except Exception as e:
        return '解码失败'
    return result_text.strip()


def html_encode(string):
    return html.escape(string)


def html_decode(string):
    return html.unescape(string)


def jsfuck_encode(source_text):
    # jsfuck_encode
    package_path = os.path.abspath(os.path.dirname(__file__))
    file_path = os.path.join(package_path, 'plugin', 'js', 'jsfuck_encode.js')
    content = execjs.compile(js_from_file(file_path))
    return content.call("JSFuck", source_text)


def jsfuck_decode(source_text):
    # jsfuck_decode
    package_path = os.path.abspath(os.path.dirname(__file__))
    file_path = os.path.join(package_path, 'plugin', 'js', 'jsfuck_decode.js')
    content = execjs.compile(js_from_file(file_path))
    return content.call("decode", source_text)


def aaencode(source_text):
    # aaencode
    package_path = os.path.abspath(os.path.dirname(__file__))
    file_path = os.path.join(package_path, 'plugin', 'js', 'aaencode.js')
    content = execjs.compile(js_from_file(file_path))
    return content.call("aaencode", source_text)


def aadecode(source_text):
    # aadecode
    package_path = os.path.abspath(os.path.dirname(__file__))
    file_path = os.path.join(package_path, 'plugin', 'js', 'aaencode.js')
    content = execjs.compile(js_from_file(file_path))
    return content.call("aadecode", source_text)


def str_to_hex(string, encoding='utf-8', byteorder='big'):
    b = string.encode(encoding)
    b = int.from_bytes(b, byteorder)
    x = hex(b)
    return x


def hex_to_str(hex_int, decoding='utf-8'):
    if type(hex_int) == str:
        hex_int = int(hex_int, 16)  # 防呆（防止输入了个字符串）
    h = hex(hex_int)
    if '0x' in str(h):
        return bytes.fromhex(h[2:]).decode(decoding)
    return bytes.fromhex(h).decode(decoding)

    """
    import zipfile

    # 打开 ZIP 文件
    with zipfile.ZipFile("myfile.zip") as zip_file:
        # 解压缩 ZIP 文件中的 file1.txt 和 file2.txt 文件
        zip_file.extractall(members=["file1.txt", "file2.txt"])
    """


def zip_unzip(filename, password=None, members=None, path=None):
    """
    :param filename: zip file path and name
    :param password: zip password, format is string!!!
    :param members: Specify the list of files to decompress
    :param path: unzip path
    :return:
    """
    if zipfile.is_zipfile(filename):
        if password:
            with zipfile.ZipFile(filename) as zip_file:
                # 如果密码被填充，则首先尝试文件是否需要密码
                try:
                    zip_file.testzip() # 验证ZIP是否需要密码
                except Exception as e:
                    # 如果密码不正确，则会引发 RunTimeError 异常
                    if isinstance(e, RuntimeError):
                        zip_file.extractall(pwd=bytes(password, "utf-8"), members=members, path=path)
                    else:
                        # 如果不需要密码，直接在下面进行解密。
                        zip_file.extractall(members=members, path=path)
                        return True
        else:
            with zipfile.ZipFile(filename) as zip_file:
                # 测试 ZIP 文件的所有文件是否可以成功解压缩
                try:
                    zip_file.testzip()
                except Exception as e:
                    # 如果密码不正确，则会引发 RunTimeError 异常
                    if isinstance(e, RuntimeError):
                        return False  # 因为没有密码，所以无法尝试解压缩操作
                    else:
                        zip_file.extractall(members=members, path=path)
                        return True


class ZipPasswordCracking:
    def __init__(self, filename, threadline=10, sleep_time=0, pass_list=None, path=None):
        self.results = None  # 存储结果
        self.zip_file = filename
        self.pass_list = pass_list
        self.threadline = threadline
        self.sleep_time = sleep_time
        self.path = path
        self.main()

    def read_pass(self):
        if self.pass_list:
            pass  # 如果使用自定义的pass_list,这里不用读取
        else:
            package_path = os.path.abspath(os.path.dirname(__file__))
            file_path = os.path.join(package_path, 'plugin', 'txt', 'zippass.txt')
            self.pass_list = read_file_to_list(file_path)

    def check_zip_is_passed(self):
        # 如果zip需要密码，则返回True，如果不需要，则返回False
        if zipfile.is_zipfile(self.zip_file):
            with zipfile.ZipFile(self.zip_file) as zip_file:
                # 测试 ZIP 文件的所有文件是否可以成功解压缩
                try:
                    zip_file.testzip()
                    return False
                except Exception as e:
                    # 如果密码不正确，则会引发 RunTimeError 异常
                    if isinstance(e, RuntimeError):
                        return True

    def crack_password(self, password):
        # 打开 ZIP 文件
        with zipfile.ZipFile(self.zip_file) as zip_file:
            # 尝试解压缩文件
            try:
                zip_file.extractall(pwd=bytes(password, "utf-8"), path=self.path)
                return True  # 密码正确
            except Exception as e:
                # 如果密码不正确，则会引发 RunTimeError 异常
                if isinstance(e, RuntimeError):
                    return False  # 密码错误

    def crack(self):
        while not self.q.empty():
            # 从队列中取出密码
            key = self.q.get()
            if self.crack_password(key):
                self.results = key
            time.sleep(self.sleep_time)
            self.q.task_done()

    def main(self):
        # 该函数作用是防止浪费线程所以进行的判断
        if self.check_zip_is_passed():
            self.run()
        else:
            self.results = "No password is required to decompress this package."

    def run(self):
        self.read_pass()
        self.q = queue.Queue()
        for password in self.pass_list:
            self.q.put(password)
        for i in range(self.threadline):
            thread = threading.Thread(target=self.crack)
            thread.start()
        self.q.join()  # Wait for thread to finish



import hashlib
import hmac


def md5(input_string):
    # MD5
    m = hashlib.md5()
    # 将输入字符串转换为字节并计算散列值
    m.update(input_string.encode())
    # 输出散列值
    return m.hexdigest()


def sha512(input_string):
    # SHA256
    m = hashlib.sha512()
    # 将输入字符串转换为字节并计算散列值
    m.update(input_string.encode())
    # 输出散列值
    return m.hexdigest()


def sha384(input_string):
    # SHA384
    m = hashlib.sha384()
    # 将输入字符串转换为字节并计算散列值
    m.update(input_string.encode())
    # 输出散列值
    return m.hexdigest()


def sha256(input_string):
    # 创建一个SHA-256对象
    m = hashlib.sha256()
    # 将输入字符串转换为字节并计算散列值
    m.update(input_string.encode())
    # 输出散列值
    return m.hexdigest()


def sha224(input_string):
    # SHA224
    m = hashlib.sha224()
    # 将输入字符串转换为字节并计算散列值
    m.update(input_string.encode())
    # 输出散列值
    return m.hexdigest()


def sha1(input_string):
    # 创建一个SHA1对象
    m = hashlib.sha1()
    # 将输入字符串转换为字节并计算散列值
    m.update(input_string.encode())
    # 输出散列值
    return m.hexdigest()


def shake_128(input_string, bits):
    # 创建一个Shake128对象
    m = hashlib.new('shake_128')
    # 将输入字符串转换为字节并计算散列值
    m.update(input_string.encode())
    # 输出散列值
    return m.hexdigest(int(bits))


def shake_256(input_string, bits):
    # 创建一个Shake128对象
    m = hashlib.new('shake_256')
    # 将输入字符串转换为字节并计算散列值
    m.update(input_string.encode())
    # 输出散列值
    return m.hexdigest(int(bits))


def HMAC_SHA256_HEX(secret, data):
    # 使用 HMAC-SHA256 算法和密钥对数据进行签名
    secret = str(secret)
    data = str(data)
    signature = hmac.new(secret.encode("utf-8"), data.encode("utf-8"), hashlib.sha256).hexdigest()
    return signature


def sha3_512(input_string):
    # sha3-256
    h = hashlib.sha3_512()
    h.update(input_string.encode())
    return h.hexdigest()


def sha3_384(input_string):
    # sha3-384
    h = hashlib.sha3_384()
    h.update(input_string.encode())
    return h.hexdigest()


def sha3_256(input_string):
    # sha3-256
    h = hashlib.sha3_256()
    h.update(input_string.encode())
    return h.hexdigest()


def sha3_224(input_string):
    # sha3-224
    h = hashlib.sha3_224()
    h.update(input_string.encode())
    return h.hexdigest()


# Base编码解码功能
# 2023年1月1日
# 末心
import base64
import qsnctf.plugin.python.python3base92
import struct
import base62
import base58


# python3base92: https://github.com/Moxin1044/Python3Base92


def base100_encode(text, encoding="utf-8", decoding="utf-8"):
    if isinstance(text, str):
        data = text.encode(encoding)
    out = [240, 159, 0, 0]*len(data)
    for i, b in enumerate(data):
        out[4*i+2] = (b + 55) // 64 + 143
        out[4*i+3] = (b + 55) % 64 + 128
    return bytes(out).decode(decoding)


def base100_decode(text, encoding="utf-8", decoding="utf-8"):
    if isinstance(text, str):
        data = text.encode(encoding)
    if len(data) % 4 != 0:
        raise Exception('Length of string should be divisible by 4')
    tmp = 0
    out = [None] * (len(data) // 4)
    for i, b in enumerate(data):
        if i % 4 == 2:
            tmp = ((b - 143) * 64) % 256
        elif i % 4 == 3:
            out[i // 4] = (b - 128 + tmp - 55) & 0xff
    return bytes(out).decode(decoding)


def base92_encode(data):
    return qsnctf.plugin.python.python3base92.b92encode(data)


def base92_decode(data):
    return qsnctf.plugin.python.python3base92.b92decode(data)


def base91_encode(data):
    data = data.encode()
    base91_alphabet = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
                       'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
                       'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
                       'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
                       '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '!', '#', '$',
                       '%', '&', '(', ')', '*', '+', ',', '.', '/', ':', ';', '<', '=',
                       '>', '?', '@', '[', ']', '^', '_', '`', '{', '|', '}', '~', '"']
    decode_table = dict((v, k) for k, v in enumerate(base91_alphabet))
    b, n = 0, 0
    out = ''
    for count in range(len(data)):
        byte = data[count:count + 1]
        b |= struct.unpack('B', byte)[0] << n
        n += 8
        if n > 13:
            v = b & 8191
            if v > 88:
                b >>= 13
                n -= 13
            else:
                v = b & 16383
                b >>= 14
                n -= 14
            out += base91_alphabet[v % 91] + base91_alphabet[v // 91]
    if n:
        out += base91_alphabet[b % 91]
        if n > 7 or b > 90:
            out += base91_alphabet[b // 91]
    return out


def base91_decode(data):
    base91_alphabet = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
                       'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
                       'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
                       'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
                       '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '!', '#', '$',
                       '%', '&', '(', ')', '*', '+', ',', '.', '/', ':', ';', '<', '=',
                       '>', '?', '@', '[', ']', '^', '_', '`', '{', '|', '}', '~', '"']
    decode_table = dict((v, k) for k, v in enumerate(base91_alphabet))
    v, b, n = -1, 0, 0
    out = bytearray()
    for strletter in data:
        if not strletter in decode_table:
            continue
        c = decode_table[strletter]
        if v < 0:
            v = c
        else:
            v += c * 91
            b |= v << n
            n += 13 if (v & 8191) > 88 else 14
            while True:
                out += struct.pack('B', b & 255)
                b >>= 8
                n -= 8
                if not n > 7:
                    break
            v = -1
    if v + 1:
        out += struct.pack('B', (b | v << n) & 255)

    return out.decode()


def base85_encode(text, encoding="utf-8", decoding="utf-8"):
    code = (base64.b85encode(text.encode(encoding))).decode(decoding)
    return code


def base85_decode(text, encoding="utf-8", decoding="utf-8"):
    code = (base64.b85decode(text.encode(encoding)))
    return code.decode(decoding)


def base64_encode(text, encoding="utf-8", decoding="utf-8"):
    code = (base64.b64encode(text.encode(encoding))).decode(decoding)
    return code


def base64_decode(text, encoding="utf-8", decoding="utf-8"):
    code = (base64.b64decode(text.encode(encoding))).decode(decoding)
    return code


def base64_encode_custom(source_text, custom_table, encoding="utf-8", decoding="utf-8"):
    try:
        STANDARD_ALPHABET = b'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'
        CUSTOM_ALPHABET = custom_table.encode(encoding)
        encode_typeTRANS = bytes.maketrans(STANDARD_ALPHABET, CUSTOM_ALPHABET)
        result_text = base64.b64encode(source_text.encode(encoding)).translate(encode_typeTRANS).decode(decoding)
        return result_text
    except Exception as e:
        return str(e)


def base64_decode_custom(source_text, custom_table, encoding="utf-8", decoding="utf-8"):
    try:
        STANDARD_ALPHABET = b'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'
        CUSTOM_ALPHABET = custom_table.encode(encoding)
        encode_typeTRANS = bytes.maketrans(STANDARD_ALPHABET, CUSTOM_ALPHABET)
        result_text = base64.b64decode(source_text.encode(encoding)).translate(encode_typeTRANS).decode(decoding)
        return result_text
    except Exception as e:
        return str(e)


def base62_encode(ints):
    return base62.encode(ints)


def base62_decode(text):
    return base62.decode(text)


def base58_encode(text, encoding="utf-8", decoding="utf-8"):
    return base58.b58encode(text.encode(encoding)).decode(decoding)


def base58_decode(text, encoding="utf-8", decoding="utf-8"):
    return base58.b58decode(text.encode(encoding)).decode(decoding)


def base36_encode(number):
    charset = "0123456789abcdefghijklmnopqrstuvwxyz"
    encoded = ""
    while number > 0:
        encoded = charset[number % 36] + encoded
        number //= 36
    if encoded == "":
        encoded = "0"
    return encoded


def base36_decode(encoded):
    charset = "0123456789abcdefghijklmnopqrstuvwxyz"
    number = 0
    for i, ch in enumerate(encoded[::-1]):
        number += charset.index(ch) * (36 ** i)
    return number


def base32_encode(text, encoding="utf-8", decoding="utf-8"):
    code = (base64.b32encode(text.encode(encoding))).decode(decoding)
    return code


def base32_decode(text, encoding="utf-8", decoding="utf-8"):
    code = (base64.b32decode(text.encode(encoding)))
    return code.decode(decoding)


def base16_encode(text, encoding="utf-8", decoding="utf-8"):
    code = (base64.b16encode(text.encode(encoding))).decode(decoding)
    return code


def base16_decode(text, encoding="utf-8", decoding="utf-8"):
    code = (base64.b16decode(text.encode(encoding)))
    return code.decode(decoding)


import qsnctf.plugin.python.operation
import requests
import json
import os


# 这里的操作一般都是需要联网的，如果是线下赛请确认主办方允许联网使用


class quipqiup:
    def __init__(self, ciphertext, clues=''):
        self.json = None
        self.text = None
        self.list = None
        # 上是三个返回值，用于取回不同的数据类型，均在quipqiup_return中定义
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
            "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
            "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
            "Origin": "https://www.quipqiup.com", "Referer": "https://www.quipqiup.com/",
            "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
            "Te": "trailers", "Connection": "close"}
        self.ciphertext = ciphertext
        self.clues = clues
        self.url = 'http://www.quipqiup.com/'
        self.id = self.quipqiup_get_id()
        self.quipqiup_return()

    def quipqiup_get_id(self):
        url = f"{self.url}solve"
        data = {"ciphertext": self.ciphertext, "clues": self.clues, "mode": "auto", "was_auto": True, "was_clue": False}
        response = requests.post(url, headers=self.headers, json=data).json()
        return response['id']

    def quipqiup_return(self):
        url = f"{self.url}status"
        data = {"id": int(self.id)}
        response_data = requests.post(url, headers=self.headers, json=data).json()
        self.json = response_data  # json 直接返回requests的response json
        return_list = []
        for response_list in response_data['solutions']:
            return_list.append(response_list['plaintext'])
        self.list = return_list
        self.text = ','.join(return_list)


class FeishuWebhook:
    # 遵循CamelCase命名
    def __init__(self, title, message, token, send_type='text'):
        """
        :param title: send_title
        :param message: send_message
        :param token: feishu_token Get the content after/v2/hook/of the url
        :param send_type: text or card
        """
        self.url = f"https://open.feishu.cn/open-apis/bot/v2/hook/{token}"
        self.title = title
        self.message = message
        self.headers = {
            "Content-Type": "application/json",
            "charset": "utf-8"
        }
        self.send_type = send_type
        self.send()

    def send(self):
        if self.send_type == 'text':
            data = {
                "msg_type": "text",
                "content": {
                    "text": f"{self.title}\n{self.message}"
                }
            }
        elif self.send_type == 'card':
            data = {
                "msg_type": "interactive",
                "card": {
                    "config": {
                        "wide_screen_mode": True,
                        "enable_forward": True
                    },
                    "elements": [{
                        "tag": "div",
                        "text": {
                            "content": self.message,
                            "tag": "lark_md"
                        }
                    }],
                    "header": {
                        "title": {
                            "content": self.title,
                            "tag": "plain_text"
                        }
                    }
                }
            }
        else:
            raise ValueError("Invalid send_type")
        data = json.dumps(data, ensure_ascii=True).encode("utf-8")
        requests.post(self.url, data=data, headers=self.headers)


class DingTalk:
    def __init__(self, title, message, token):
        """
        :param title: send_title
        :param message: send_message
        :param token: dingding tolken
        """
        self.url = f"https://oapi.dingtalk.com/robot/send?access_token={token}"
        self.title = title
        self.message = message
        self.headers = {
            "Content-Type": "application/json",
            "charset": "utf-8"
        }
        self.send()

    def send(self):
        data = {
            "msgtype": "text",
            "text":
                {
                    "content": f"{self.title}\n{self.message}"
                }
        }
        data = json.dumps(data, ensure_ascii=True).encode("utf-8")
        requests.post(self.url, data=data, headers=self.headers)


class ThreatBook:
    def __init__(self, api_key):
        self.key = api_key

    def ip_reputation(self, ip):
        api = "https://api.threatbook.cn/v3/scene/ip_reputation"
        data = {
            'apikey': self.key,
            'resource': ip,
            'lang': 'zh'
        }
        q = requests.post(url=api, data=data)
        """ 引用
        q_data = json.loads(q.text)
        if q_data['response_code'] == 0:
            data = q_data['data']
            IP_info = data[IP]
            severity = IP_info['severity']  # 威胁等级
            IP_judg = IP_info['judgments']
            if len(IP_judg) != 1:
                tags = ""
                for tag in IP_judg:
                    tags = tags+tag+","
                tags = tags[:-1]
            else:
                tags = IP_judg[0]  # IP标签
            basic = IP_info['basic']
            carrier = basic['carrier'] # 运营商
            location = basic['location']
            country = location['country']  # 国家
            province = location['province']  # 省
            city = location['city']  # 城市
            lng = location['lng']  # 经度
            lat = location['lat']  # 纬度
            scene = IP_info['scene']  # 应用场景
            confidence_level = IP_info['confidence_level']  # 可信度
            is_malicious = IP_info['is_malicious']  # 是否为恶意
            update_time = IP_info['update_time']  # 更新时间
            tags_classes = IP_info['tags_classes']
            if len(tags_classes) != 1:
                tagss = ""
                for tagsss in tags_classes:
                    tagss = tagss + tagsss + ","
                tagss = tagss[:-1]  # 团伙标签
            else:
                tagss = "空"
        """
        return q.json()

    def file_upload(self, file_path, file_name, sandbox_type='win7_sp1_enx64_office2013'):
        url = 'https://api.threatbook.cn/v3/file/upload';
        fields = {
            'apikey': self.key,
            'sandbox_type': sandbox_type,
            'run_time': 60
        }
        file_dir = file_path
        file_name = file_name
        files = {
            'file': (file_name, open(os.path.join(file_dir, file_name), 'rb'))
        }
        response = requests.post(url, data=fields, files=files)
        return response.json()

    def file_report_multiengines(self, sha256):
        """
        :param sha256: file_sha256
        :return: {'data': {'multiengines': {'threat_level': 'clean', 'total': 22,
        'is_white': False, 'total2': 22, 'positives': 0, 'scan_date': '2023-01-05 19:04:42', 'scans': {'IKARUS':
        'safe', 'vbwebshell': 'safe', 'Avast': 'safe', 'Avira': 'safe', 'Sophos': 'safe', 'K7': 'safe',
        'Rising': 'safe', 'Kaspersky': 'safe', 'Panda': 'safe', 'Baidu-China': 'safe', 'NANO': 'safe',
        'Antiy': 'safe', 'AVG': 'safe', 'Baidu': 'safe', 'DrWeb': 'safe', 'GDATA': 'safe', 'Microsoft': 'safe',
        'Qihu360': 'safe', 'ESET': 'safe', 'ClamAV': 'safe', 'JiangMin': 'safe', 'Trustlook': 'safe'}}},
        'response_code': 0, 'verbose_msg': 'OK'}
        """
        url = 'https://api.threatbook.cn/v3/file/report/multiengines'
        params = {
            'apikey': self.key,
            'sha256': sha256
        }
        response = requests.get(url, params=params)
        return response.json()

    def file_report(self, sha256, sandbox_type='win7_sp1_enx64_office2013'):
        """
        :param sha256: file_sha256
        :param sandbox_type: win7_sp1_enx64_office2013
        :return:
        """
        url = 'https://api.threatbook.cn/v3/file/report'
        params = {
            'apikey': self.key,
            'sandbox_type': sandbox_type,
            'sha256': sha256
        }
        response = requests.get(url, params=params)
        return response.json()


class FOFA:
    def __init__(self, email, key, proxy=""):
        self.username = None
        self.email_check = None
        self.email = email
        self.key = key
        self.url = 'https://fofa.info/api/v1'
        self.proxy = proxy
        self.get_userinfo()

    def check_fofa_config(self):
        return f"Email:{self.email} Key:{self.key} Proxy:{self.proxy}"

    def get_userinfo(self):
        # Check Email and key
        url = f"{self.url}/info/my?email={self.email}&key={self.key}"
        response = qsnctf.plugin.python.operation.send_get_json(url, self.proxy)
        if response['error']:
            return response['errmsg']
        else:
            self.email_check = response['email']
            self.username = response['username']
            self.isvip = response['isvip']
            self.viplevel = response['vip_level']
            self.avatar = response['avatar']
            self.fcoin = response['fcoin']
            return self

    def userinfo(self):
        # Check Email and key
        url = f"{self.url}/info/my?email={self.email_check}&key={self.key}"
        response = qsnctf.plugin.python.operation.send_get_json(url, self.proxy)
        if response['error']:
            return response['errmsg']
        else:
            return response

    def search(self, query_text, field=None, page=1, size=100, full=False):
        if field is None:
            field = ['ip', 'host', 'port']
        fields = ','.join(field)
        query = qsnctf.plugin.python.operation.get_base64_url(query_text)
        url = f"{self.url}/search/all?email={self.email_check}&key={self.key}&qbase64={query}&fields={fields}&page={page}&size={size}&full={full}"
        response = qsnctf.plugin.python.operation.send_get_json(url, self.proxy)
        '''
            # 考虑到生产环境，所以不可以在这里直接返回errmsg，统一返回response即可。
            # 下同
            if response['error']:
                return response['errmsg']
            else:
                return response
            '''
        return response

    def search_stats(self, query_text, field=None):
        if field is None:
            field = ['title']
        fields = ','.join(field)
        query = qsnctf.plugin.python.operation.get_base64_url(query_text)
        url = f"{self.url}/search/stats?fields={fields}&qbase64={query}&email={self.email_check}&key={self.key}"
        response = qsnctf.plugin.python.operation.send_get_json(url, self.proxy)
        return response

    def search_host(self, host, detail=False):
        url = f"{self.url}/host/{host}?detail={detail}&email={self.email_check}&key={self.key}"
        response = qsnctf.plugin.python.operation.send_get_json(url, self.proxy)
        return response


class DaSheng:
    def __init__(self, id, key):
        """
        :param id: id
        :param key: key # https://sandbox.freebuf.com/cloudApi
        """
        self.id = id
        self.key = key

    def token(self):
        api = "https://sandbox.riskivy.com/openapi/oauth/token"
        data = {
            'client_id': self.id,
            'client_secret': self.key,
            'grant_type': 'client_credentials',
            'scope': 'openapi'
        }
        q = requests.post(url=api, data=data)
        data = q.json()
        access_token = data['access_token']
        return access_token

    def upload(self, file_dir, file_name):
        api = "https://sandbox.riskivy.com/openapi/mac/sample/upload"
        headers = {
            'Authorization': 'Bearer ' + self.token()
        }
        files = {
            'file': (open(os.path.join(file_dir, file_name), 'rb'))
        }
        response = requests.post(api, headers=headers, files=files)
        # q = requests.post(url=api, headers=headers)
        return response.json()

    def search(self, sha1):
        api = "https://sandbox.riskivy.com/openapi/mac/sample/report/" + sha1
        headers = {
            'Authorization': 'Bearer ' + self.token()
        }
        response = requests.get(api, headers=headers)
        return response.json()


class ZeroZeon:
    def __init__(self, key):
        self.key = key

    def search(self, title):
        url = "https://0.zone/api/data/"
        payload = json.dumps({
            "title": title,
            "title_type": "site",
            "page": 1,
            "pagesize": 10,
            "zone_key_id": self.key
        })
        headers = {
            'User-Agent': 'qsnCTF/1.0.0 (https://www.qsnctf.com)',
            'Content-Type': 'application/json',
            'Accept': '*/*',
            'Host': '0.zone',
            'Connection': 'keep-alive'
        }
        response = requests.request("POST", url, headers=headers, data=payload)
        return response.json()


class GoCQHttp:
    def __init__(self, url, auth=False, authorization=''):
        self.url = url
        self.auth = auth
        self.Authorization = authorization
        if auth:
            self.headers = {
                "Authorization": self.Authorization
            }
        else:
            self.headers = ""

    def send_private_msg(self, user_id, message):
        """
        :param user_id: QQ
        :param message: message text
        :return:
        """
        data = {
            "user_id": user_id,
            "message": message
        }
        q = requests.post(self.url + "/send_private_msg", data=data, headers=self.headers)
        return q.json()

    def send_group_msg(self, group_id, message):
        """
        :param group_id: QQ group ID
        :param message: message text
        :return:
        """
        data = {
            "group_id": group_id,
            "message": message
        }
        q = requests.post(self.url + "/send_group_msg", data=data, headers=self.headers)
        return q.json()
